/* Data Write and Read test for all Ns (using PIO) */

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <stdio.h>
#include "pcc.h"

//---------They are what they are here....-----------
#define FRAME_LEN 100
#define BUF_LEN FRAME_LEN*2
//--------------------------------------------


void main(int argc, char **argv)
{
//================================================================
//	At first, check or tell how to use this Application
//================================================================
	int Nmax=24;
	//---------argment check and help------------
	if ( argc== 2 )
	{
		Nmax = atol(argv[1]);
		Nmax++;
	}

//================================================================
//	Second, Do various initialization...
//================================================================
	//--------------- at first. CAMAC OPEN---------------
	int fd;
	if ( (fd=cam_open())<0 )
	{
		puts("CAMAC open failed\n");
		return;
	}

	//--------------- FIFO should be cleared(Non-mandatry)---------------
	if ( cam_clear_fifo(fd) )
	{
		puts("CAMAC clear failed\n");
		cam_close(fd);
		return;
	}


//================================================================
//	Preparaitons have finished! Then, input data and obtain data.
//================================================================

	//-------------N,A,F,W input---------------
	int N=1, A=0, F=0x10, W=0xFFFFFF;
	
	int length = FRAME_LEN;
	int cmdbuf[BUF_LEN+2], rplybuf[BUF_LEN+2];

	//------------- Preparing for buffer(before "cam_gen_cc", DO THIS ABSOLUTELY!!)---------------
	if ( cam_gen_init(length, cmdbuf) )
	{
		puts("COMMAND buffer init failed\n");
	}
	if ( cam_gen_init(length, rplybuf) )
	{
		puts("REPLY buffer init failed\n");
	}

	//-----Sequential-frame(=Packet) generation-----
	do
	{
		//---CAMAC WRITE operation---
		if ( cam_gen_cc( cmdbuf, N, A, F, W) )
		{
			printf("cam_gen_cc failed @ N=%d, F=0x%x\n", N, F);
			cam_close(fd);
			return;
		}
		
		if ( F )
		{
			F=0x00;
		}
		else
		{
			F=0x10;
			N++;
		}
	}
	while (N<Nmax);
		

	//----------Execution----------
	if ( cam_exec_pio(fd, cmdbuf, rplybuf) )
	{
		printf("cam_exec_pio failed");
		cam_close(fd);
		return;
	}
	
	int act_len;
	int sent_dat[BUF_LEN+2], got_dat[BUF_LEN+2];
	if ( cam_extract_cc_data(cmdbuf, BUF_LEN, &act_len, sent_dat) )
	{
		printf("Nmax-1=%d\tact_len=%d\n", Nmax-1, act_len);
		puts("CMD data extraction failed\n");
		cam_close(fd);
		return;
	}

	if ( cam_extract_cc_data(rplybuf, BUF_LEN, &act_len, got_dat) )
	{
		puts("RPL data extraction failed\n");
		cam_close(fd);
		return;
	}
	

	//---WR and RD validation..---
	int i=0, sent_index, got_index;
	do
	{
		sent_index = i;
		got_index = i+1;

//		printf("sent=%06x\tgot=%06x\n", sent_dat[i-1], got_dat[i-1]);
		
		
		if ( sent_dat[sent_index] != got_dat[got_index] )
		{
			printf("WR and RD failed @ N=%d\n",i/2+1);
		}
		else
		{
			printf("\tWR and RD OK! @ N=%d\n",i/2+1);
		}
		i+=2;
	}
	while ( i < act_len);



//================================================================
//	termination of this programing Operation
//================================================================
	//------------- CAMAC close---------------
	cam_close( fd );
	puts("Done");

	return;
}
